"""
Pose Module for estimating the pose of the body

Author: Roshaan Mehmood
"""
import cv2
import mediapipe as mp
import time
import math


class poseDetector():

    def __init__(self, mode=False, model=1, smooth=True, enable_seg=False,
                 detectionCon=0.5, trackCon=0.5):

        self.mode = mode
        self.model = model
        self.smooth = smooth
        self.enable_seg = enable_seg
        self.detectionCon = detectionCon
        self.trackCon = trackCon

        self.mpDraw = mp.solutions.drawing_utils
        self.mpPose = mp.solutions.pose
        self.pose = self.mpPose.Pose(self.mode, self.model, self.smooth, self.enable_seg,
                                     self.detectionCon, self.trackCon)

    def findPose(self, img, draw=True):
        """
        Finds the pose in a BGR image or from a webcam.
        :param img: Image to find the faces to detect.
        :param draw: Flag to draw the pose connections.
        :return: Landmarks List.
        """
        imgRGB = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        self.results = self.pose.process(imgRGB)
        if self.results.pose_landmarks:
            if draw:
                self.mpDraw.draw_landmarks(img, self.results.pose_landmarks,
                                           self.mpPose.POSE_CONNECTIONS)
        return img

    def findPosition(self, img, draw=True):
        """
        Finds the position of pose in a BGR image or from a webcam.
        :param img: Image to find the faces to detect.
        :param draw: Flag to draw the pose connections.
        :return: Landmarks List.
        """
        self.lmList = []
        if self.results.pose_landmarks:
            for id, lm in enumerate(self.results.pose_landmarks.landmark):
                h, w, c = img.shape
                # print(id, lm)
                cx, cy = int(lm.x * w), int(lm.y * h)
                self.lmList.append([id, cx, cy])
                if draw:
                    cv2.circle(img, (cx, cy), 5, (255, 0, 0), cv2.FILLED)
        return self.lmList

    def findAngle(self, img, p1, p2, p3, draw=True):
        """
        Finds the angle between two specified poses.
        :param img: Image to find the faces to detect.
        :param p1: 1st point of the pose.
        :param p2: 2nd point of the pose.
        :param p3: 3rd point of the pose.
        :param draw: Flag to draw the pose connections.
        :return: Landmarks List.
        """
        # Get the landmarks
        x1, y1 = self.lmList[p1][1:]
        x2, y2 = self.lmList[p2][1:]
        x3, y3 = self.lmList[p3][1:]

        # Calculate the Angle
        angle = math.degrees(math.atan2(y3 - y2, x3 - x2) -
                             math.atan2(y1 - y2, x1 - x2))
        if angle < 0:
            angle += 360

        # print(angle)

        # Draw
        if draw:
            cv2.line(img, (x1, y1), (x2, y2), (255, 255, 255), 3)
            cv2.line(img, (x3, y3), (x2, y2), (255, 255, 255), 3)
            cv2.circle(img, (x1, y1), 10, (0, 0, 255), cv2.FILLED)
            cv2.circle(img, (x1, y1), 15, (0, 0, 255), 2)
            cv2.circle(img, (x2, y2), 10, (0, 0, 255), cv2.FILLED)
            cv2.circle(img, (x2, y2), 15, (0, 0, 255), 2)
            cv2.circle(img, (x3, y3), 10, (0, 0, 255), cv2.FILLED)
            cv2.circle(img, (x3, y3), 15, (0, 0, 255), 2)
            cv2.putText(img, str(int(angle)), (x2 - 50, y2 + 50),
                        cv2.FONT_HERSHEY_PLAIN, 2, (0, 0, 255), 2)
        return angle


def main():
    cap = cv2.VideoCapture(0)
    pTime = 0
    detector = poseDetector()
    while True:
        success, img = cap.read()
        img = detector.findPose(img, draw=False)
        lmList = detector.findPosition(img, draw=False)
        if len(lmList) != 0:
            print(lmList[14])
            cv2.circle(img, (lmList[14][1], lmList[14][2]), 15, (0, 0, 255), cv2.FILLED)

        cTime = time.time()
        fps = 1 / (cTime - pTime)
        pTime = cTime

        cv2.putText(img, str(int(fps)), (70, 50), cv2.FONT_HERSHEY_PLAIN, 3,
                    (255, 0, 0), 3)

        cv2.imshow("Image", img)
        cv2.waitKey(1)


if __name__ == "__main__":
    main()



import cv2
import numpy as np


class ObjectDetector:
    def __init__(self, weightpath, configpath, labelsPath):
        self.weightpath = weightpath
        self.configpath = configpath
        self.net = cv2.dnn_DetectionModel(self.weightpath, self.configpath)
        self.net.setInputSize(320, 320)
        self.net.setInputScale(1.0 / 127.5)
        self.net.setInputMean((127.5, 127.5, 127.5))
        self.net.setInputSwapRB(True)
        self.labelsPath = labelsPath
        if self.labelsPath:
            label_file = open(self.labelsPath, 'rt')
            self.classLabels = []
            for line in label_file:
                stripped_line = line.strip()
                self.classLabels.append(stripped_line)
            label_file.close()
        else:
            print("No Labels Found")

    def DetectObject(self, img, thres=0.45, nms_threshold=0.2, color=(0, 255, 0)):
        img = cv2.flip(img, 1)
        classIds, confs, bbox = self.net.detect(img, confThreshold=0.5)
        bbox = list(bbox)
        confs = list(np.array(confs).reshape(1, -1)[0])
        confs = list(map(float, confs))

        indices = cv2.dnn.NMSBoxes(bbox, confs, thres, nms_threshold)

        for i in indices:
            i = i[0]
            box = bbox[i]
            x, y, w, h = box[0], box[1], box[2], box[3]
            cv2.rectangle(img, (x, y), (x + w, h + y), color=color, thickness=2)
            cv2.putText(img, self.classLabels[classIds[i][0] - 1].upper(), (box[0] + 10, box[1] + 30),
                        cv2.FONT_HERSHEY_SIMPLEX, 1, color, 2)
        return img, indices

    # if len(classIds) != 0:
    #     for classId, confidence, box in zip(classIds.flatten(), confs.flatten(), bbox):
    #         cv2.rectangle(img, box, color=(0, 255, 0), thickness=2)
    #         cv2.putText(img, classLabels[classId - 1].upper(), (box[0] + 10, box[1] + 30),
    #                     cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 255, 0), 2)
    #         cv2.putText(img, str(round(confidence * 100, 2)), (box[0] + 200, box[1] + 30),
    #                     cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 255, 0), 2)


def main():
    cap = cv2.VideoCapture(0)
    cap.set(3, 640)
    cap.set(4, 480)

    configpath = 'ssd_mobilenet_v3_large_coco_2020_01_14.pbtxt'
    weightpath = 'frozen_inference_graph.pb'
    labelsPath = 'coco.names'

    objectdetector = ObjectDetector(weightpath, configpath, labelsPath)

    while True:
        success, img = cap.read()
        img = cv2.flip(img, 1)
        img, indices = objectdetector.DetectObject(img)
        print(indices)
        cv2.imshow("Output", img)
        cv2.waitKey(1)


if __name__ == '__main__':
    main()



import tensorflow.keras
import numpy as np
import cv2


class Classifier:

    def __init__(self, modelPath, labelsPath=None):
        self.model_path = modelPath
        # Disable scientific notation for clarity
        np.set_printoptions(suppress=True)
        # Load the model
        self.model = tensorflow.keras.models.load_model(self.model_path)

        # Create the array of the right shape to feed into the keras model
        # The 'length' or number of images you can put into the array is
        # determined by the first position in the shape tuple, in this case 1.
        self.data = np.ndarray(shape=(1, 224, 224, 3), dtype=np.float32)
        self.labels_path = labelsPath
        if self.labels_path:
            label_file = open(self.labels_path, "r")
            self.list_labels = []
            for line in label_file:
                stripped_line = line.strip()
                self.list_labels.append(stripped_line)
            label_file.close()
        else:
            print("No Labels Found")

    def getPrediction(self, img, draw=True, pos=(50, 50), scale=2, color=(0, 255, 0)):
        # resize the image to a 224x224 with the same strategy as in TM2:
        imgS = cv2.resize(img, (224, 224))
        # turn the image into a numpy array
        image_array = np.asarray(imgS)
        # Normalize the image
        normalized_image_array = (image_array.astype(np.float32) / 127.0) - 1

        # Load the image into the array
        self.data[0] = normalized_image_array

        # run the inference
        prediction = self.model.predict(self.data)
        indexVal = np.argmax(prediction)

        if draw and self.labels_path:
            cv2.putText(img, str(self.list_labels[indexVal]),
                        pos, cv2.FONT_HERSHEY_COMPLEX, scale, color, 2)

        return list(prediction[0]), indexVal


def main():
    cap = cv2.VideoCapture(0)
    maskClassifier = Classifier('MyModel/keras_model.h5', 'MyModel/labels.txt')
    while True:
        _, img = cap.read()
        predection = maskClassifier.getPrediction(img)
        print(predection)
        cv2.imshow("Image", img)
        cv2.waitKey(1)


if __name__ == "__main__":
    main()



import serial
import time
import logging
import serial.tools.list_ports


class SerialObject:
    """
    Allow to transmit data to a Serial Device like Arduino.
    Example send $255255000
    """

    def __init__(self, portNo=None, baudRate=9600, digits=1):
        """
        Initialize the serial object.
        :param portNo: Port Number.
        :param baudRate: Baud Rate.
        :param digits: Number of digits per value to send
        """
        self.portNo = portNo
        self.baudRate = baudRate
        self.digits = digits
        connected = False
        if self.portNo is None:
            ports = list(serial.tools.list_ports.comports())
            for p in ports:
                if "Arduino" in p.description:
                    print(f'{p.description} Connected')
                    self.ser = serial.Serial(p.device)
                    self.ser.baudrate = baudRate
                    connected = True
            if not connected:
                logging.warning("Arduino Not Found. Please enter COM Port Number instead.")

        else:
            try:
                self.ser = serial.Serial(self.portNo, self.baudRate)
                print("Serial Device Connected")
            except:
                logging.warning("Serial Device Not Connected")

    def sendData(self, data):
        """
        Send data to the Serial device
        :param data: list of values to send
        """
        myString = "$"
        for d in data:
            myString += str(int(d)).zfill(self.digits)
        try:
            self.ser.write(myString.encode())
            return True
        except:
            return False

    def getData(self):
        """
        :param numOfVals: number of vals to retrieve
        :return: list of data received
        """
        data = self.ser.readline()
        data = data.decode("utf-8")
        data = data.split('#')
        dataList = []
        [dataList.append(d) for d in data]
        return dataList[:-1]


def main():
    arduino = SerialObject()
    while True:
        arduino.sendData([1, 1, 1, 1, 1])
        time.sleep(2)
        arduino.sendData([0, 0, 0, 0, 0])
        time.sleep(2)


if __name__ == "__main__":
    main()



"""
Hand Tracking Module

Author: Roshaan Mehmood
"""

import cv2
import mediapipe as mp
import math


class HandDetector:
    """
    Finds Hands using the mediapipe library. Exports the landmarks
    in pixel format. Adds extra functionalities like finding how
    many fingers are up or the distance between two fingers. Also
    provides bounding box info of the hand found.
    """

    def __init__(self, mode=False, maxHands=2, minDetConf=0.5, minTrackCon=0.5):
        """
        :param mode: In static mode, detection is done on each image: slower
        :param maxHands: Maximum number of hands to detect
        :param minDetConf: Minimum Detection Confidence Threshold
        :param minTrackCon: Minimum Tracking Confidence Threshold
        """
        self.mode = mode
        self.maxHands = maxHands
        self.minDetConf = minDetConf
        self.minTrackCon = minTrackCon

        self.mpHands = mp.solutions.hands
        self.hands = self.mpHands.Hands(static_image_mode=self.mode, max_num_hands=self.maxHands,
                                        min_detection_confidence=self.minDetConf,
                                        min_tracking_confidence=self.minTrackCon)
        self.mpDraw = mp.solutions.drawing_utils
        self.tipIds = [4, 8, 12, 16, 20]
        self.fingers = []
        self.lmList = []

    def findHands(self, img, drawConns=True, flipType=True, bBox=False,
                  bBoxColor=(255, 0, 255), showHandType=False, textColor=(255, 0, 255)):
        """
        Finds hands in a BGR image.

        :param img: Image to find the hands in.
        :param drawConns: Flag to draw the hand connections.
        :param flipType: Flag to flip the hand type.
        :param bBox: Flag to draw the bounding box around the hand.
        :param bBoxColor: Color of bounding box around the hand.
        :param showHandType: Flag to show type of hand (left or right).
        :param textColor: Color of text of hand type.
        :return: Image with or without drawings.
        """
        imgRGB = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        self.results = self.hands.process(imgRGB)
        allHands = []
        h, w, c = img.shape
        if self.results.multi_hand_landmarks:
            for handType, handLms in zip(self.results.multi_handedness, self.results.multi_hand_landmarks):
                myHand = {}
                ## lmList
                mylmList = []
                xList = []
                yList = []
                for id, lm in enumerate(handLms.landmark):
                    px, py, pz = int(lm.x * w), int(lm.y * h), int(lm.z * w)
                    mylmList.append([px, py, pz])
                    xList.append(px)
                    yList.append(py)

                ## bbox
                xmin, xmax = min(xList), max(xList)
                ymin, ymax = min(yList), max(yList)
                boxW, boxH = xmax - xmin, ymax - ymin
                bbox = xmin, ymin, boxW, boxH
                cx, cy = bbox[0] + (bbox[2] // 2), \
                         bbox[1] + (bbox[3] // 2)

                myHand["lmList"] = mylmList
                myHand["bbox"] = bbox
                myHand["center"] = (cx, cy)

                if flipType:
                    if handType.classification[0].label == "Right":
                        myHand["type"] = "Left"
                    else:
                        myHand["type"] = "Right"
                else:
                    myHand["type"] = handType.classification[0].label
                allHands.append(myHand)

                if drawConns:
                    self.mpDraw.draw_landmarks(img, handLms,
                                               self.mpHands.HAND_CONNECTIONS)
                if bBox:
                    cv2.rectangle(img, (bbox[0] - 20, bbox[1] - 20),
                                  (bbox[0] + bbox[2] + 20, bbox[1] + bbox[3] + 20),
                                  bBoxColor, 2)
                if showHandType:
                    cv2.putText(img, myHand["type"], (bbox[0] - 30, bbox[1] - 30), cv2.FONT_HERSHEY_PLAIN,
                                2, textColor, 2)
        if drawConns:
            return allHands, img
        else:
            return allHands

    def fingersUp(self, myHand):
        """
        Finds how many fingers are open and returns in a list.
        Considers left and right hands separately
        :return: List of which fingers are up
        """
        myHandType = myHand["type"]
        myLmList = myHand["lmList"]
        if self.results.multi_hand_landmarks:
            fingers = []
            # Thumb
            if myHandType == "Right":
                if myLmList[self.tipIds[0]][0] > myLmList[self.tipIds[0] - 1][0]:
                    fingers.append(1)
                else:
                    fingers.append(0)
            else:
                if myLmList[self.tipIds[0]][0] < myLmList[self.tipIds[0] - 1][0]:
                    fingers.append(1)
                else:
                    fingers.append(0)

            # 4 Fingers
            for id in range(1, 5):
                if myLmList[self.tipIds[id]][1] < myLmList[self.tipIds[id] - 2][1]:
                    fingers.append(1)
                else:
                    fingers.append(0)
        return fingers

    def findDistance(self, p1, p2, img=None):
        """
        Find the distance between two landmarks based on their
        index numbers.
        :param p1: Point1
        :param p2: Point2
        :param img: Image to draw on.
        :param draw: Flag to draw the output on the image.
        :return: Distance between the points
                 Image with output drawn
                 Line information
        """

        x1, y1 = p1
        x2, y2 = p2
        cx, cy = (x1 + x2) // 2, (y1 + y2) // 2
        length = math.hypot(x2 - x1, y2 - y1)
        info = (x1, y1, x2, y2, cx, cy)
        if img is not None:
            cv2.circle(img, (x1, y1), 15, (255, 0, 255), cv2.FILLED)
            cv2.circle(img, (x2, y2), 15, (255, 0, 255), cv2.FILLED)
            cv2.line(img, (x1, y1), (x2, y2), (255, 0, 255), 3)
            cv2.circle(img, (cx, cy), 15, (255, 0, 255), cv2.FILLED)
            return length, info, img
        else:
            return length, info


def main():
    cap = cv2.VideoCapture(0)
    detector = HandDetector(minDetConf=0.8, maxHands=1)
    while True:
        # Get image frame
        success, img = cap.read()
        # Find the hand and its landmarks
        hands, img = detector.findHands(img)  # with draw
        # hands = detector.findHands(img, drawConns=False)  # without draw

        if hands:
            # Hand 1
            hand1 = hands[0]
            lmList1 = hand1["lmList"]  # List of 21 Landmark points
            bbox1 = hand1["bbox"]  # Bounding box info x,y,w,h
            centerPoint1 = hand1['center']  # center of the hand cx,cy
            handType1 = hand1["type"]  # Handtype Left or Right

            fingers1 = detector.fingersUp(hand1)
            print(fingers1)

            # if len(hands) == 2:
            #     # Hand 2
            #     hand2 = hands[1]
            #     lmList2 = hand2["lmList"]  # List of 21 Landmark points
            #     bbox2 = hand2["bbox"]  # Bounding box info x,y,w,h
            #     centerPoint2 = hand2['center']  # center of the hand cx,cy
            #     handType2 = hand2["type"]  # Hand Type "Left" or "Right"
            #
            #     fingers2 = detector.fingersUp(hand2)
            #
            #     # Find Distance between two Landmarks. Could be same hand or different hands
            #     length, info, img = detector.findDistance(lmList1[8][0:2], lmList2[8][0:2], img)  # with draw
            #     # length, info = detector.findDistance(lmList1[8], lmList2[8])  # with draw
        # Display
        cv2.imshow("Image", img)
        cv2.waitKey(1)


if __name__ == "__main__":
    main()




"""
Face Mesh Module

Author: Roshaan Mehmood
"""

import cv2
import mediapipe as mp
import math


class FaceMeshDetector:
    """
    Face Mesh Detector to find 468 Landmarks using the mediapipe library.
    Helps acquire the landmark points in pixel format
    """

    def __init__(self, staticMode=False, maxFaces=2, minDetConf=0.5, minTrackConf=0.5,
                 lineThick=1, circleRadius=2):
        """
        :param staticMode: In static mode, detection is done on each image: slower
        :param maxFaces: Maximum number of faces to detect
        :param minDetConf: Minimum Detection Confidence Threshold
        :param minTrackConf: Minimum Tracking Confidence Threshold
        :param lineThick: Thickness of the mesh connection lines
        :param circleRadius: Radius of the circle of Face Mesh
        """
        self.staticMode = staticMode
        self.maxFaces = maxFaces
        self.minDetConf = minDetConf
        self.minTrackConf = minTrackConf
        self.lineThick = lineThick
        self.circleRadius = circleRadius

        self.mpDraw = mp.solutions.drawing_utils
        self.mpFaceMesh = mp.solutions.face_mesh
        self.faceMesh = self.mpFaceMesh.FaceMesh(static_image_mode=self.staticMode,
                                                 max_num_faces=self.maxFaces,
                                                 min_detection_confidence=self.minDetConf,
                                                 min_tracking_confidence=self.minTrackConf)
        self.drawSpec = self.mpDraw.DrawingSpec(thickness=lineThick, circle_radius=circleRadius)

    def findFaceMesh(self, img, drawConns=True):
        """
        Finds face landmarks in BGR Image.

        :param img: Image to find the face landmarks in.
        :param drawConns: Flag to draw the output on the image.
        :return: Image with or without drawings
        """
        self.imgRGB = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        self.results = self.faceMesh.process(self.imgRGB)
        faces = []
        if self.results.multi_face_landmarks:
            for faceLms in self.results.multi_face_landmarks:
                if drawConns:
                    self.mpDraw.draw_landmarks(img, faceLms, self.mpFaceMesh.FACEMESH_CONTOURS,
                                               self.drawSpec, self.drawSpec)
                face = []
                for id, lm in enumerate(faceLms.landmark):
                    ih, iw, ic = img.shape
                    x, y = int(lm.x * iw), int(lm.y * ih)
                    face.append([x, y])
                faces.append(face)
        return img, faces

    def findDistance(self, p1, p2, img=None):
        """
        Find the distance between two landmarks based on their
        index numbers.

        :param p1: Point1
        :param p2: Point2
        :param img: Image to draw on.
        :return: Distance between the points
                 Image with output drawn
                 Line information
        """

        x1, y1 = p1
        x2, y2 = p2
        cx, cy = (x1 + x2) // 2, (y1 + y2) // 2
        length = math.hypot(x2 - x1, y2 - y1)
        info = (x1, y1, x2, y2, cx, cy)
        if img is not None:
            cv2.circle(img, (x1, y1), 8, (255, 0, 255), cv2.FILLED)
            cv2.circle(img, (x2, y2), 8, (255, 0, 255), cv2.FILLED)
            cv2.line(img, (x1, y1), (x2, y2), (255, 0, 255), 3)
            cv2.circle(img, (cx, cy), 8, (255, 0, 255), cv2.FILLED)
            return length,info, img
        else:
            return length, info


def main():
    cap = cv2.VideoCapture(0)
    detector = FaceMeshDetector(maxFaces=2)
    while True:
        success, img = cap.read()
        img, faces = detector.findFaceMesh(img)
        if faces:
            print(faces[0])
        cv2.imshow("Image", img)
        cv2.waitKey(1)


if __name__ == "__main__":
    main()



"""
A Library concluding the necessary Functions for Computer Vision using OpenCV
Author: Roshaan Mehmood
"""

import cv2
import numpy as np
import copy


def imgFlip(img, horFlip=False):
    """
    function to flip an image vertically or horizontally. It is based on openCV flip function

    :param img: Image to flip in.
    :param horFlip: Flag for flipping the image horizontally.
    :return: Flipped Image.
    """
    if horFlip:
        img = cv2.flip(img, 0)
    else:
        img = cv2.flip(img, 1)

    return img


def imagesStack(imageList, cols, scale):
    """
    Stack Images together to display in a single window.

    :param imageList: list of images to stack
    :param cols: the num of img in a row
    :param scale: bigger~1+ ans smaller~1-
    :return: Stacked Image
    """
    imageList = copy.deepcopy(imageList)

    # make the array full by adding blank img, otherwise the openCV can't work
    totalImages = len(imageList)
    rows = totalImages // cols if totalImages // cols * cols == totalImages else totalImages // cols + 1
    blankImages = cols * rows - totalImages

    width = imageList[0].shape[1]
    height = imageList[0].shape[0]
    imgBlank = np.zeros((height, width, 3), np.uint8)
    imageList.extend([imgBlank] * blankImages)

    # resize the images
    for i in range(cols * rows):
        imageList[i] = cv2.resize(imageList[i], (0, 0), None, scale, scale)
        if len(imageList[i].shape) == 2:
            imageList[i] = cv2.cvtColor(imageList[i], cv2.COLOR_GRAY2BGR)

    # put the images in a board
    hor = [imgBlank] * rows
    for y in range(rows):
        line = []
        for x in range(cols):
            line.append(imageList[y * cols + x])
        hor[y] = np.hstack(line)
    ver = np.vstack(hor)
    return ver


def cornerRect(img, bbox, drawRect=True, corLen=30, corThick=2, rectThick=1,
               rectColor=(255, 255, 255), corColor=(255, 0, 255)):
    """
    Draw the rectangle with corners on image

    :param img: Image to draw rectangle on.
    :param bbox: Bounding box [x, y, w, h]
    :param drawRect: Flag to draw the rectangle
    :param corLen: Length of rectangle corners
    :param corThick: thickness of rectangle corners
    :param rectThick: thickness of the rectangle
    :param rectColor: Color of the Rectangle
    :param corColor: Color of rectangle corners
    :return: Image with rectangle
    """
    x, y, w, h = bbox
    x1, y1 = x + w, y + h
    if rectThick != 0 and drawRect:
        cv2.rectangle(img, bbox, rectColor, rectThick)
    # Top Left  x,y
    cv2.line(img, (x, y), (x + corLen, y), corColor, corThick)
    cv2.line(img, (x, y), (x, y + corLen), corColor, corThick)
    # Top Right  x1,y
    cv2.line(img, (x1, y), (x1 - corLen, y), corColor, corThick)
    cv2.line(img, (x1, y), (x1, y + corLen), corColor, corThick)
    # Bottom Left  x,y1
    cv2.line(img, (x, y1), (x + corLen, y1), corColor, corThick)
    cv2.line(img, (x, y1), (x, y1 - corLen), corColor, corThick)
    # Bottom Right  x1,y1
    cv2.line(img, (x1, y1), (x1 - corLen, y1), corColor, corThick)
    cv2.line(img, (x1, y1), (x1, y1 - corLen), corColor, corThick)

    return img


def findContours(img, imgPre, minArea=1000, sort=True, filter=0, drawContours=True, color=(255, 0, 0)):
    """
    Finds Contours in an image.

    :param img: Image on which we want to draw
    :param imgPre: Image on which we want to find contours
    :param minArea: Minimum Area to detect as valid contour
    :param sort: True will sort the contours by area (biggest first)
    :param filter: Filters based on the corner points e.g. 4 = Rectangle or square
    :param drawContours: draw contours boolean
    :param color: Color of the contours and shapes
    :return: Foudn contours with [contours, Area, BoundingBox, Center]
    """
    conFound = []
    imgContours = img.copy()
    contours, hierarchy = cv2.findContours(imgPre, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_NONE)

    for cnt in contours:
        area = cv2.contourArea(cnt)
        if area > minArea:
            peri = cv2.arcLength(cnt, True)
            approx = cv2.approxPolyDP(cnt, 0.02 * peri, True)
            # print(len(approx))
            if len(approx) == filter or filter == 0:
                if drawContours: cv2.drawContours(imgContours, cnt, -1, color, 3)
                x, y, w, h = cv2.boundingRect(approx)
                cx, cy = x + (w // 2), y + (h // 2)
                cv2.rectangle(imgContours, (x, y), (x + w, y + h), color, 2)
                cv2.circle(imgContours, (x + (w // 2), y + (h // 2)), 5, color, cv2.FILLED)
                conFound.append({"cnt": cnt, "area": area, "bbox": [x, y, w, h], "center": [cx, cy]})

    if sort:
        conFound = sorted(conFound, key=lambda x: x["area"], reverse=True)

    return imgContours, conFound


def PNGOverlay(imgBack, imgFront, pos=[0, 0]):
    hf, wf, cf = imgFront.shape
    hb, wb, cb = imgBack.shape
    *_, mask = cv2.split(imgFront)
    maskBGRA = cv2.cvtColor(mask, cv2.COLOR_GRAY2BGRA)
    maskBGR = cv2.cvtColor(mask, cv2.COLOR_GRAY2BGR)
    imgRGBA = cv2.bitwise_and(imgFront, maskBGRA)
    imgRGB = cv2.cvtColor(imgRGBA, cv2.COLOR_BGRA2BGR)

    imgMaskFull = np.zeros((hb, wb, cb), np.uint8)
    imgMaskFull[pos[1]:hf + pos[1], pos[0]:wf + pos[0], :] = imgRGB
    imgMaskFull2 = np.ones((hb, wb, cb), np.uint8) * 255
    maskBGRInv = cv2.bitwise_not(maskBGR)
    imgMaskFull2[pos[1]:hf + pos[1], pos[0]:wf + pos[0], :] = maskBGRInv

    imgBack = cv2.bitwise_and(imgBack, imgMaskFull2)
    imgBack = cv2.bitwise_or(imgBack, imgMaskFull)

    return imgBack


def imageRotate(img, angle, scale=1):
    h, w = img.shape[:2]
    center = (w / 2, h / 2)
    rotate_matrix = cv2.getRotationMatrix2D(center=center, angle=angle, scale=scale)
    img = cv2.warpAffine(src=img, M=rotate_matrix, dsize=(w, h))
    return img


def putRectText(img, text, pos, scale=3, textThick=3, textColor=(255, 255, 255),
                rectColor=(255, 0, 255), font=cv2.FONT_HERSHEY_PLAIN,
                offset=10, border=None, borderColor=(0, 255, 0)):
    """
    Creates Text with Rectangle Background.

    :param img: Image to put text rect on
    :param text: Text inside the rectangle
    :param pos: Starting position of the rect x1,y1
    :param scale: Scale of the text
    :param textThick: Thickness of the text
    :param textColor: Color of the Text
    :param rectColor: Color of the Rectangle
    :param font: Font used. Must be cv2.FONT....
    :param offset: Clearance around the text
    :param border: Outline around the rect
    :param borderColor: Color of the outline
    :return: image, rect (x1,y1,x2,y2)
    """
    ox, oy = pos
    (w, h), _ = cv2.getTextSize(text, font, scale, textThick)

    x1, y1, x2, y2 = ox - offset, oy + offset, ox + w + offset, oy - h - offset

    cv2.rectangle(img, (x1, y1), (x2, y2), rectColor, cv2.FILLED)
    if border is not None:
        cv2.rectangle(img, (x1, y1), (x2, y2), borderColor, border)
    cv2.putText(img, text, (ox, oy), font, scale, textColor, textThick)

    return img, [x1, y2, x2, y1]


def main():
    cap = cv2.VideoCapture(0)
    while True:
        success, img = cap.read()
        img, bbox = putRectText(img, "pycamdetector", [50, 50], 2, 2, offset=10, border=5)
        imgGray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
        imageList = [img, img, imgGray, img, imgGray]
        imgStacked = imagesStack(imageList, 2, 0.5)

        cv2.imshow("Stacked Images", imgStacked)
        cv2.waitKey(1)


if __name__ == "__main__":
    main()



import cv2
import numpy as np
import time
import math


class LivePlot:
    def __init__(self, width=640, height=480, yLimit=[0, 100],
                 interval=0.001, invert=True, char=' '):

        self.yLimit = yLimit
        self.width = width
        self.height = height
        self.invert = invert
        self.interval = interval
        self.char = char[0]
        self.imgPlot = np.zeros((self.height, self.width, 3), np.uint8)
        self.imgPlot[:] = 225, 225, 225

        cv2.rectangle(self.imgPlot, (0, 0),
                      (self.width, self.height),
                      (0, 0, 0), cv2.FILLED)
        self.xP = 0
        self.yP = 0

        self.yList = []

        self.xList = [x for x in range(0, 100)]
        self.ptime = 0

    def update(self, y, lineColor=(255, 0, 255)):

        if time.time() - self.ptime > self.interval:

            # Refresh
            self.imgPlot[:] = 225, 225, 225
            # Draw Static Parts
            self.drawBackground()
            # Draw the text value
            cv2.putText(self.imgPlot, str(y),
                        (self.width - (125), 50), cv2.FONT_HERSHEY_PLAIN,
                        3, (150, 150, 150), 3)
            if self.invert:
                self.yP = int(np.interp(y, self.yLimit,
                                        [self.height, 0]))
            else:
                self.yP = int(np.interp(y, self.yLimit,
                                        [0, self.height]))
            self.yList.append(self.yP)
            if len(self.yList) == 100:
                self.yList.pop(0)

            for i in range(0, len(self.yList)):
                if i < 2:
                    pass
                else:
                    cv2.line(self.imgPlot, (int((self.xList[i - 1] * (self.width // 100))) - (self.width // 10),
                                            self.yList[i - 1]),
                             (int((self.xList[i] * (self.width // 100)) - (self.width // 10)),
                              self.yList[i]), lineColor, 2)
            self.ptime = time.time()

        return self.imgPlot

    def drawBackground(self):
        # Draw Background Canvas
        cv2.rectangle(self.imgPlot, (0, 0),
                      (self.width, self.height),
                      (0, 0, 0), cv2.FILLED)

        # Center Line
        cv2.line(self.imgPlot, (0, self.height // 2), (self.width, self.height // 2), (150, 150, 150), 2)

        # Draw Grid Lines
        for x in range(0, self.width, 50):
            cv2.line(self.imgPlot, (x, 0), (x, self.height),
                     (50, 50, 50), 1)

        for y in range(0, self.height, 50):
            cv2.line(self.imgPlot, (0, y), (self.width, y),
                     (50, 50, 50), 1)
            #  Y Label
            cv2.putText(self.imgPlot,
                        f'{int(self.yLimit[1] - ((y / 50) * ((self.yLimit[1] - self.yLimit[0]) / (self.height / 50))))}',
                        (10, y), cv2.FONT_HERSHEY_PLAIN,
                        1, (150, 150, 150), 1)
        cv2.putText(self.imgPlot, self.char,
                    (self.width - 100, self.height - 25), cv2.FONT_HERSHEY_PLAIN,
                    5, (150, 150, 150), 5)


def main():
    xPlot = LivePlot(width=1200, yLimit=[-100, 100], interval=0.01)
    x = 0
    while True:

        x += 1
        if x == 360: x = 0
        imgPlot = xPlot.update(int(math.sin(math.radians(x)) * 100))

        cv2.imshow("Image", imgPlot)
        if cv2.waitKey(1) & 0xFF == ord('q'):
            break


if __name__ == "__main__":
    main()



from pycamdetector.FaceDetectionModule import FaceDetector
from pycamdetector.HandTrackingModule import HandDetector
from pycamdetector.FaceMeshModule import FaceMeshDetector
from pycamdetector.PlotModule import LivePlot
from pycamdetector.PoseModule import poseDetector
from pycamdetector.ObjectDetectionModule import ObjectDetector
from pycamdetector.FPS import FPS
from pycamdetector.Utils import imgFlip, imagesStack, cornerRect, findContours, \
    PNGOverlay, imageRotate, putRectText
from pycamdetector.ClassificationModule import Classifier
from pycamdetector.ColorModule import ColorFinder
from pycamdetector.SerialModule import SerialObject



import time
import cv2


class FPS:
    """
    Helps in finding Frames Per Second and display on an OpenCV Image
    """

    def __init__(self):
        self.pTime = time.time()

    def showFPS(self, img=None, textPos=(20, 50), textColor=(255, 255, 255), textScale=3, textThick=3):
        """
        Update the frame rate.

        :param img: Image to display on, can be left blank if only fps value required
        :param textPos: Position of the FPS on the image
        :param textColor: Color of the FPS Value displayed
        :param textScale: Scale of the FPS Value displayed
        :param textThick: Thickness of the FPS Value displayed
        :return: Image with FPS Value
        """
        cTime = time.time()
        try:
            fps = 1 / (cTime - self.pTime)
            self.pTime = cTime
            if img is None:
                return fps
            else:
                cv2.putText(img, f'FPS: {int(fps)}', textPos, cv2.FONT_HERSHEY_PLAIN,
                            textScale, textColor, textThick)
                return fps, img
        except:
            return 0


def main():
    # """
    # Without Webcam
    # """
    # fpsReader = FPS()
    # while True:
    #     time.sleep(0.025)  # add delay to get 40 Frames per second
    #     fps = fpsReader.showFPS()
    #     print(fps)

    """
    With Webcam
    """
    FPSReader = FPS()
    cap = cv2.VideoCapture(0)
    while True:
        success, img = cap.read()
        fps, img = FPSReader.showFPS(img)
        cv2.imshow("Image", img)
        cv2.waitKey(1)


if __name__ == "__main__":
    main()



"""
Face Detection Module

Author: Roshaan Mehmood
"""
import cv2
import mediapipe as mp


class FaceDetector:
    """
        Find faces in realtime using the light weight model provided in the mediapipe
        library.
    """
    def __init__(self, minDetConf=0.5):
        """
            :param minDetConf: Minimum Detection Confidence Threshold
        """
        self.minDetConf = minDetConf
        self.mp_face_detection = mp.solutions.face_detection
        self.mp_drawing = mp.solutions.drawing_utils
        self.face_detection = self.mp_face_detection.FaceDetection(self.minDetConf)

    def findFaces(self, img, drawRect=True, showPerc=True, textColor=(255, 255, 255),
                  rectColor=(255, 255, 255), rectThick=1, corLen=30, corColor=(255, 0, 255), corThick=2):
        """
        Finds faces to detect in a BGR image.

        :param img: Image to find the faces in.
        :param drawRect: Flag to draw the rectangle around the faces detected.
        :param showPerc: Flag to display accuracy percentage.
        :param textColor: Color of the text of accuracy percentage.
        :param rectColor: Color of the rectangle.
        :param rectThick: Thickness of the rectangle.
        :param corLen: Length of the corner lines of rectangle.
        :param corColor: Color of the rectangle corners.
        :param corThick: Thickness of the rectangle corners.
        :return: Image with or without drawings.
                 Bounding Box list.
        """
        imgRGB = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        self.results = self.face_detection.process(imgRGB)
        bboxs = []

        if self.results.detections:
            for detection in self.results.detections:
                bboxC = detection.location_data.relative_bounding_box
                ih, iw, ic = img.shape
                bbox = int(bboxC.xmin * iw), int(bboxC.ymin * ih), int(bboxC.width * iw), int(bboxC.height * ih)
                bboxs.append([bbox, detection.score])
                if drawRect:
                    img = self.fancyDraw(img, bbox, rectColor=rectColor, corColor=corColor,
                                         t=corThick, rt=rectThick, l=corLen)
                if showPerc:
                    cv2.putText(img, f' {int(detection.score[0] * 100)}%',
                                (bbox[0], bbox[1] - 20), cv2.FONT_HERSHEY_PLAIN, 2, textColor, 2)
        return img, bboxs

    def fancyDraw(self, img, bbox, l=30, t=2, rt=1, rectColor=(255, 255, 255), corColor=(255, 0, 255)):
        x, y, w, h = bbox
        x1, y1 = x + w, y + h
        cv2.rectangle(img, bbox, rectColor, rt)
        # Top left x,y
        cv2.line(img, (x, y), (x + l, y), corColor, t)
        cv2.line(img, (x, y), (x, y + l), corColor, t)

        # Top right x1,y
        cv2.line(img, (x1, y), (x1 - l, y), corColor, t)
        cv2.line(img, (x1, y), (x1, y + l), corColor, t)

        # Bottom left x,y1
        cv2.line(img, (x, y1), (x + l, y1), corColor, t)
        cv2.line(img, (x, y1), (x, y1 - l), corColor, t)

        # Bottom Right x1,y1
        cv2.line(img, (x1, y1), (x1 - l, y1), corColor, t)
        cv2.line(img, (x1, y1), (x1, y1 - l), corColor, t)

        return img


def main():
    cap = cv2.VideoCapture(0)
    detector = FaceDetector(minDetConf=0.85)
    while True:
        success, img = cap.read()
        img = cv2.cv2.flip(img, 1)
        img, bboxs = detector.findFaces(img, showPerc=False)
        print(bboxs)
        cv2.imshow('Image', img)
        if cv2.waitKey(1) & 0xFF == ord('q'):
            break


if __name__ == "__main__":
    main()



"""
Color Module
Finds color in an image based on hsv values
Can run as stand alone to find relevant hsv values

"""

import cv2
import numpy as np
import logging


class ColorFinder:
    def __init__(self, trackBar=False):
        self.trackBar = trackBar
        if self.trackBar:
            self.initTrackbars()

    def empty(self, a):
        pass

    def initTrackbars(self):
        """
        To intialize Trackbars . Need to run only once
        """
        cv2.namedWindow("TrackBars")
        cv2.resizeWindow("TrackBars", 640, 240)
        cv2.createTrackbar("Hue Min", "TrackBars", 0, 179, self.empty)
        cv2.createTrackbar("Hue Max", "TrackBars", 179, 179, self.empty)
        cv2.createTrackbar("Sat Min", "TrackBars", 0, 255, self.empty)
        cv2.createTrackbar("Sat Max", "TrackBars", 255, 255, self.empty)
        cv2.createTrackbar("Val Min", "TrackBars", 0, 255, self.empty)
        cv2.createTrackbar("Val Max", "TrackBars", 255, 255, self.empty)

    def getTrackbarValues(self):
        """
        Gets the trackbar values in runtime
        :return: hsv values from the trackbar window
        """
        hmin = cv2.getTrackbarPos("Hue Min", "TrackBars")
        smin = cv2.getTrackbarPos("Sat Min", "TrackBars")
        vmin = cv2.getTrackbarPos("Val Min", "TrackBars")
        hmax = cv2.getTrackbarPos("Hue Max", "TrackBars")
        smax = cv2.getTrackbarPos("Sat Max", "TrackBars")
        vmax = cv2.getTrackbarPos("Val Max", "TrackBars")

        hsvVals = {"hmin": hmin, "smin": smin, "vmin": vmin,
                   "hmax": hmax, "smax": smax, "vmax": vmax}
        print(hsvVals)
        return hsvVals

    def update(self, img, myColor=None):
        """
        :param img: Image in which color needs to be found
        :param hsvVals: List of lower and upper hsv range
        :return: (mask) bw image with white regions where color is detected
                 (imgColor) colored image only showing regions detected
        """
        imgColor = [],
        mask = []

        if self.trackBar:
            myColor = self.getTrackbarValues()

        if isinstance(myColor, str):
            myColor = self.getColorHSV(myColor)

        if myColor is not None:
            imgHSV = cv2.cvtColor(img, cv2.COLOR_BGR2HSV)
            lower = np.array([myColor['hmin'], myColor['smin'], myColor['vmin']])
            upper = np.array([myColor['hmax'], myColor['smax'], myColor['vmax']])
            mask = cv2.inRange(imgHSV, lower, upper)
            imgColor = cv2.bitwise_and(img, img, mask=mask)
        return imgColor, mask

    def getColorHSV(self, myColor):

        if myColor == 'red':
            output = {'hmin': 146, 'smin': 141, 'vmin': 77, 'hmax': 179, 'smax': 255, 'vmax': 255}
        elif myColor == 'green':
            output = {'hmin': 44, 'smin': 79, 'vmin': 111, 'hmax': 79, 'smax': 255, 'vmax': 255}
        elif myColor == 'blue':
            output = {'hmin': 103, 'smin': 68, 'vmin': 130, 'hmax': 128, 'smax': 255, 'vmax': 255}
        else:
            output = None
            logging.warning("Color Not Defined")
            logging.warning("Available colors: red, green, blue ")

        return output


def main():
    myColorFinder = ColorFinder(False)
    cap = cv2.VideoCapture(0)
    cap.set(3, 640)
    cap.set(4, 480)

    # Custom Orange Color
    hsvVals = {'hmin': 10, 'smin': 55, 'vmin': 215, 'hmax': 42, 'smax': 255, 'vmax': 255}

    while True:
        success, img = cap.read()
        imgRed, _ = myColorFinder.update(img, "red")
        imgGreen, _ = myColorFinder.update(img, "green")
        imgBlue, _ = myColorFinder.update(img, "blue")
        imgOrange, _ = myColorFinder.update(img, hsvVals)

        cv2.imshow("Red", imgRed)
        cv2.imshow("Image", img)
        if cv2.waitKey(1) & 0xFF == ord('q'):
            break


if __name__ == "__main__":
    main()
